import cv2

from pyrecorder.recorder import Recorder


class Streamer(Recorder):

    def __init__(self,
                 mode="live",
                 delay=None,
                 close_window=True) -> None:
        super().__init__()
        self.close_window = close_window
        self.frame = None

        if mode == "live":
            self.delay = 1
        elif mode == "key":
            self.delay = 0
        elif mode == "stream":
            self.delay = delay
        else:
            self.delay = None

    def do(self, frame):
        cv2.imshow('image', frame)
        if self.delay is not None:
            cv2.waitKey(self.delay)

        self.frame = frame

    def close(self):
        if self.close_window:
            cv2.destroyAllWindows()
        else:
            delay = self.delay
            self.delay = 0
            self.do(self.frame)
            self.delay = delay





from IPython.display import Image
from PIL import Image

from pyrecorder.recorder import Recorder


class GIF(Recorder):

    def __init__(self,
                 fname,
                 optimize=False,
                 duration=500,
                 loop=0
                 ):
        super().__init__()
        self.fname = fname
        self.frames = []
        self.optimize = optimize
        self.duration = duration
        self.loop = loop

    def do(self, frame):
        img = Image.fromarray(frame, 'RGB')
        self.frames.append(img)

    def close(self):
        self.frames[0].save(self.fname,
                            save_all=True,
                            append_images=self.frames[1:],
                            optimize=self.optimize,
                            duration=self.duration,
                            loop=self.loop)




import io

import cv2
import matplotlib.pyplot as plt
import numpy as np

from pyrecorder.converter import Converter


class Matplotlib(Converter):

    def __init__(self, dpi=100, close_after_recording=True) -> None:
        super().__init__()
        self.dpi = dpi
        self.close_after_recording = close_after_recording

    def do(self):
        buf = io.BytesIO()
        plt.savefig(buf, format='png', dpi=self.dpi)

        buf.seek(0)
        _bytes = np.asarray(bytearray(buf.read()), dtype=np.uint8)
        img = cv2.imdecode(_bytes, cv2.IMREAD_COLOR)

        if self.close_after_recording:
            plt.close()

        return img



import warnings

from pyrecorder.converters.matpotlib import Matplotlib


class Video:

    def __init__(self,
                 recorders,
                 converter=Matplotlib(),
                 ) -> None:

        super().__init__()
        self.converter = converter

        self.rec = recorders
        if not (isinstance(self.rec, list) or isinstance(self.rec, tuple)):
            self.rec = [self.rec]

    def record(self):
        img = self.converter.do()
        self.converter.finalize()

        for r in self.rec:
            r.do(img)

    def close(self):
        for r in self.rec:
            r.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self.close()

    @classmethod
    def from_iteratable(cls, video, iteratable, func=None, **kwargs):
        for entry in iteratable:
            plot = True
            if func is not None:
                ret = func(entry, **kwargs)
                plot = not (isinstance(ret, bool) and not ret)

            if plot:
                video.record()

        video.close()


def load(fname, height="500px"):
    if fname.endswith("gif"):
        from IPython.display import Image
        return Image(filename=fname)

    elif fname.endswith("mp4"):
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            from IPython.display import HTML
            code = f"<iframe height='{height}' width='100%' src='{fname}' frameborder='0' allowfullscreen></iframe>"
            return HTML(code)



from abc import abstractmethod


class Converter:

    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    def do(self):
        pass

    def finalize(self, **kwargs):
        pass




from abc import abstractmethod


class Recorder:

    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    def do(self, frame):
        pass

    def close(self):
        pass

    def load(self):
        pass



import cv2

from pyrecorder.recorder import Recorder


class File(Recorder):

    def __init__(self,
                 fname,
                 codec=cv2.VideoWriter_fourcc('m', 'p', '4', 'v'),
                 fps=1,
                 ) -> None:
        super().__init__()

        self.fname = fname
        self.codec = codec
        self.fps = fps

        self.file = None

    def do(self, frame, n_frames=1):

        if self.file is None:
            height, width, layers = frame.shape
            self.file = cv2.VideoWriter(self.fname, self.codec, self.fps, frameSize=(width, height))

        for k in range(n_frames):
            self.file.write(frame)

    def close(self):
        cv2.destroyAllWindows()
        if self.file is not None:
            self.file.release()



import setuptools
from setuptools import setup

from pyrecorder.version import __version__

# ---------------------------------------------------------------------------------------------------------
# GENERAL
# ---------------------------------------------------------------------------------------------------------


__name__ = "pyrecorder"
__author__ = "Julian Blank"
__url__ = "https://www.egr.msu.edu/coinlab/blankjul/pyrecorder/"

data = dict(
    name=__name__,
    version=__version__,
    author=__author__,
    url=__url__,
    python_requires='>=3.6',
    author_email="blankjul@egr.msu.edu",
    description="Video Recording made easy - The tool you need for animations using Matplotlib, Plotly and other "
                "plotting libraries. ",
    license='Apache License 2.0',
    keywords="video,recording,matplotlib,plotly",
    install_requires=['numpy>=1.15', 'opencv-python>=4.0.0.21', 'matplotlib>=3', 'Pillow'],
    include_package_data=True,
    platforms='any',
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Multimedia :: Video'
    ]
)


# ---------------------------------------------------------------------------------------------------------
# OTHER METADATA
# ---------------------------------------------------------------------------------------------------------


# update the readme.rst to be part of setup
def readme():
    with open('README.rst') as f:
        return f.read()


def packages():
    return ["pyrecorder"] + ["pyrecorder." + e for e in setuptools.find_packages(where='pyrecorder')]


data['long_description'] = readme()
data['packages'] = packages()

# ---------------------------------------------------------------------------------------------------------
# SETUP
# ---------------------------------------------------------------------------------------------------------

setup(**data)
